/********************************************************************
 * kcam_mb.c       19-FEB-1992            Y.Takeuchi (T.I.T.)       *
 *                                                                  *
 * FOR K2917 list processing                                        *
 * Master Block Transfer to SUN memory (DVMA+DMA)                   *
 * for SPARC cc device driver  K2917  K3922                         *
 *                                                                  *
 * 13-JUL-1992 Y.Takeuchi    use "done interrupt" to detect EOT     *
 * 30-NOV-1992 Y.Takeuchi                                           *
 *  5-MAR-1993 Y.Takeuchi    modify for unified UNIX camlib         *
 ********************************************************************/
static void ccminphys(bp)
register struct buf *bp;
{
#ifdef DEBUG
    if (Debug_level >= 2) {
        uprintf("ccminphys:bp->b_bcount=%d\n", bp->b_bcount);
    }
#endif
    if (bp->b_bcount > CC_MINPHYS_SIZE)
	bp->b_bcount = CC_MINPHYS_SIZE;
}


static void ccstrategy(bp)
register struct buf *bp;
{
    register struct cc_device *cc = &ccdevice[0];
    register struct mb_device *md = ccdinfo[0];
    register struct K_REG *k = cc->k;
    register u_long dma_addr;
    register mode = cc->mode;
    register naf  = cc->naf;
    register int wc = (mode & BIT16) ? cc->len : cc->len * 2;
    int s;

    while(!(bp->b_mbinfo = mb_mapalloc(md->md_hd->mh_map, bp, 0, 0, 0))) {
	s = spl5();
	timeout(wakeup, cc, hz / 10 + 1);
	sleep((caddr_t)cc, CCHPRI);
	splx(s);
    }
    dma_addr = (u_long)MBI_ADDR(bp->b_mbinfo);
    cc->retlen = 0;
    k->csr = RST;                             /* K2917 Reset */

    if (cc->klist == CC_KLIST_NO) {
	k->cma = CMA_INIT;                    /* Initialize memory pointer */
	k->cmr = mode | (cc->cur_crate << 8);
	k->cmr = naf;
	k->cmr = -(cc->len & 0xFFFF);             /* Max len = 1MWord */
	k->cmr = 0xFFFF;
	k->cmr = HALT;
	k->cma = CMA_INIT;                    /* Reset memory pointer */
	k->maclo = dma_addr & 0xFFFF;         /* Set DMA base address */
	k->machi = dma_addr >> 16;
	k->amr = AMR_INIT;                    /* Set VME AM code */
	k->mtc = wc;
	k->cser = DMA_RESET;                  /* DMA reset */

	switch (cc->naf & 0x0018) {
	  case 0x0000:                        /* CAMAC read */
	    k->docr = DOCR_INIT | DMA_READ;
	    k->sccr = DMA_START;
	    k->csr |= DMA;                    /* DMA mode */
	    k->csr &= ~WRITE;
	    break;

	  case 0x0010:                        /* CAMAC write */
	    k->docr = DOCR_INIT | DMA_WRITE;
	    k->sccr = DMA_START;
	    k->csr |= DMA;                    /* DMA mode */
	    k->csr |= WRITE;
	    break;

	  default:                            /* ERROR */
	    return;
	    break;
	}
    }
    else {
	k->cma = now_cma;                    /* Initialize memory pointer */
	k->maclo = dma_addr & 0xFFFF;         /* Set DMA base address */
	k->machi = dma_addr >> 16;
	k->amr = AMR_INIT;                    /* Set VME AM code */
	k->mtc = now_wc;
	k->cser = DMA_RESET;                  /* DMA reset */

	k->docr = DOCR_INIT | DMA_READ;
	k->sccr = DMA_START;
	k->csr |= DMA;                    /* DMA mode */
	k->csr &= ~WRITE;
    }

    s = spl5();
    timeout(cctimeout, cc, CC_TIMEOUT_DMA * hz);
    k->csr |= GO;                         /* Go! */
    k->donc = INT_AUTO_CLEAR | INT_ENABLE | (md->md_intpri & 7);
    sleep((caddr_t)cc, CCHPRI);
    k->donc = INT_AUTO_CLEAR | (md->md_intpri & 7);
    splx(s);

    cc->camac_qx = k->csr;

    /* Q-STOP mode or no error */ 
    if ((mode & QSCAN) == 0 || (k->csr & ERR) == 0) {
	if ((mode & BIT16) == 0) {
	    cc->retlen = cc->len - k->mtc / 2; /* retlen=number of transfer */
	    if ((mode & QSCAN) == 0 && (k->csr & ERR) != 0) /* Q-STOP mode */
	      cc->retlen = cc->retlen - 2;     /* delete the data of Q=0 */
	    cc->ptr_udata += cc->retlen * 2;   /* ptr(2bytes),retlen(4bytes) */
	    cc->len_udata += cc->retlen * 2;
	}
	else {
	    cc->retlen = cc->len - k->mtc;
	    if ((mode & QSCAN) == 0 && (k->csr & ERR) != 0) /* Q-STOP mode */
	      cc->retlen--;                    /* delete the data of Q=0 */
	    cc->ptr_udata += cc->retlen;       /* ptr(2bytes),retlen(2bytes) */
	    cc->len_udata += cc->retlen;
	}
    }

    if (cc->interrupt & CC_INT_TIMEOUT ) {
	cc->interrupt &= ~CC_INT_TIMEOUT;
	cc->status = CC_STA_BLOCK_TIMEOUT;
    }

    mb_mapfree(md->md_hd->mh_map, &bp->b_mbinfo);
    iodone(bp);

    return;
}


camac_b(cc, mode, naf, len, retlen)
struct cc_device *cc;
u_short mode, naf;
int len, *retlen;
{
    register dev_t dev = cc->dev;
    register struct uio *uio = cc->uio;
    register struct buf *bp = &ccbuf;

    cc->klist = CC_KLIST_NO;

    cc->mode = mode;
    cc->naf = naf;
    cc->len = len;

    uio->uio_iov->iov_base = (caddr_t)cc->ptr_udata;
    uio->uio_iov->iov_len  = (mode & BIT16) ? len * 2: len * 4;
    uio->uio_iovcnt = 1;
    uio->uio_offset = 0;

    if ((naf & 0x0018) == 0)
        physio(ccstrategy, bp, dev, B_READ, ccminphys, uio);
    else;
        physio(ccstrategy, bp, dev, B_WRITE, ccminphys, uio);

    *retlen = cc->retlen;

    cc_sys_status = bp->b_error;
    return cc->status;
}
