/************************************************************************
 *	camt1.c       	 30-NOV-1991		Y.Takeuchi (TIT)	*
 *									*
 *	CAMAC single action speed test					*
 *	CAMAC Scaler ch1: input 1MHz pulse				*
 *	CAMAC Switch register						*
 *	camt1 [loop_count [list_file_name]]				*
 ************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include "camtest.h"
#include "camlib.h"

#define DEFLOP    10
#define DEFNAME   "camt1.obj"
#define MAXLOOP   100000
#define MAXLIST   2000

main(argc, argv)
int argc;
char *argv[];
{
    int i, loop, nafct, nafrt, nafw, nafr, nafn, q, x, time;
    int dumy, dat0, dat1, datw = 0xffffffff;
    u_short sdumy, sdat0, sdat1, sdatw = 0xffff;
    int lenlis, data[MAXLOOP];
    unsigned short list[MAXLIST], *datas;
    struct cc_iosb iosb;
    char filename[33];

/* Read options */
    loop = (argc > 1) ? atoi(argv[1]) : DEFLOP;
    strcpy(filename, (argc > 2) ? argv[2] : DEFNAME);

/* Initialize valiable */
    nafct = NAF(CAMAC_SCALER, 1, 9);
    nafrt = NAF(CAMAC_SCALER, 1, 0);
    nafw = NAF(CAMAC_SWITCH, 0, 16);
    nafr = NAF(CAMAC_SWITCH, 0, 0);
    nafn = NAF(CAMAC_SWITCH, 0, 26);
    iosb.s_reg = loop;

/* Open and Initialize CAMAC */
    if (CAMOPN()) {
      perror("CAMOPN");
      exit(1);
  }
    CSETCR(0);
    CGENC();
    CGENZ();
    CREMI();

/* Read and Install CAMAC list */
    if (CRLIST(filename, list, MAXLIST, &lenlis, &i)) {
	perror("CRLIST");
        exit(2);
    }
    if (CLLOAD(list, lenlis, &i)) {
	perror("CLLOAD");
        exit(3);
    }

/**** Measurement ****/
/* 24bit time=0 */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
	CAMAC(nafrt, &dat1, &q, &x);
	printf("%d\n", dat1 - dat0);
    }

/* 24bit write */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
	CAMAC(nafw,  &datw, &q, &x);
	CAMAC(nafrt, &dat1, &q, &x);
	printf("%d\n", dat1 - dat0);
    }

/* 24bit read */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
	CAMAC(nafr,  &dumy, &q, &x);
	CAMAC(nafrt, &dat1, &q, &x);
	printf("%d\n", dat1 - dat0);
    }

/* 24bit ndt */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
	CAMAC(nafn,  &dumy, &q, &x);
	CAMAC(nafrt, &dat1, &q, &x);
	printf("%d\n", dat1 - dat0);
    }

/* 16bit time=0 */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMACW(nafct, &sdumy, &q, &x);
	CAMACW(nafrt, &sdat0, &q, &x);
	CAMACW(nafrt, &sdat1, &q, &x);
	printf("%d\n", sdat1 - sdat0);
    }

/* 16bit write */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMACW(nafct, &sdumy, &q, &x);
	CAMACW(nafrt, &sdat0, &q, &x);
	CAMACW(nafw,  &sdatw, &q, &x);
	CAMACW(nafrt, &sdat1, &q, &x);
	printf("%d\n", sdat1 - sdat0);
    }

/* 16bit read */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMACW(nafct, &sdumy, &q, &x);
	CAMACW(nafrt, &sdat0, &q, &x);
	CAMACW(nafr,  &sdumy, &q, &x);
	CAMACW(nafrt, &sdat1, &q, &x);
	printf("%d\n", sdat1 - sdat0);
    }

/* 16bit ndt */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMACW(nafct, &sdumy, &q, &x);
	CAMACW(nafrt, &sdat0, &q, &x);
	CAMACW(nafn,  &sdumy, &q, &x);
	CAMACW(nafrt, &sdat1, &q, &x);
	printf("%d\n", sdat1 - sdat0);
    }


/* Execute list */
    if (CEWAI(data, MAXLOOP, &iosb)) {
	perror("CEWAI");
        exit(10);
    }
/*
    datas = (unsigned short *)&data[0];
    for (i = 0; i < iosb.ret_length; i++)
      printf("%d ",datas[i]);
    printf("\n%d %d %d\n", iosb.s_reg, iosb.ret_length, iosb.status);
    exit(0);
*/

/* 24bit time=0 */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", data[(i*2)+1] - data[i*2]);

/* 24bit write */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", data[loop*2+(i*2)+1] - data[loop*2+i*2]);

/* 24bit read */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", data[loop*4+(i*2)+1] - data[loop*4+i*2]);

    datas = (unsigned short *)&data[loop*6];

/* 16bit time=0 */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", datas[(i*2)+1] - datas[i*2]);

/* 16bit write */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", datas[loop*2+(i*2)+1] - datas[loop*2+i*2]);

/* 16bit read */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", datas[loop*4+(i*2)+1] - datas[loop*4+i*2]);

/* 16bit ndt */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", datas[loop*6+(i*2)+1] - datas[loop*6+i*2]);

/* End */
    CAM_Close();
}



