/************************************************************************
 *	camt2.c       	  2-DEC-1991		Y.Takeuchi (TIT)	*
 *									*
 *	CAMAC block action speed test					*
 *	CAMAC Scaler ch1: input 1MHz pulse				*
 *	CAMAC Switch register						*
 *	camt2 [min [max [step [list_file_name]]]]			*
 ************************************************************************/
#include <stdio.h>
#include "camtest.h"
#include "camlib.h"

#define DEFMIN    100
#define DEFMAX    1000
#define DEFSTEP   100
#define DEFNAME   "camt2.obj"
#define DEFLOOP   100
#define MAXLOOP   100000
#define MAXLIST   2000
#define MAXDATA   100000

main(argc, argv)
int argc;
char *argv[];
{
    int min, max, step, loop, len, status, retlen, err;
    int i, j, nafct, nafrt, nafw, nafr, nafn, q, x, time;
    int dumy, dat0, dat1, datw = 0xffffffff;
    int lenlis, data[MAXDATA], buf[MAXDATA];
    unsigned short list[MAXLIST], datas[MAXDATA], bufs[MAXDATA], *usp;
    struct cc_iosb iosb;
    char filename[33];

/* Read options */
    min = (argc > 1) ? atoi(argv[1]) : DEFMIN;
    max = (argc > 2) ? atoi(argv[2]) : DEFMAX;
    step = (argc > 3) ? atoi(argv[3]) : DEFSTEP;
    strcpy(filename, (argc > 4) ? argv[4] : DEFNAME);
    loop = DEFLOOP;

/* Initialize valiable */
    nafct = NAF(CAMAC_SCALER, 1, 9);
    nafrt = NAF(CAMAC_SCALER, 1, 0);
    nafw = NAF(CAMAC_SWITCH, 0, 16);
    nafr = NAF(CAMAC_SWITCH, 0, 0);
    nafn = NAF(CAMAC_SWITCH, 0, 26);
    for (i = 0; i < MAXDATA; i++) {
	data[i] = i;
	datas[i] = i;
    }
    len = 0;
    for (i = min; i < max; i += step)
      len++;

/* Open and Initialize CAMAC */
    if (CAMOPN()) {
      perror("CAMOPN");
      exit(1);
  }
    CSETCR(0);
    CGENC();
    CGENZ();
    CREMI();

/* Read and Install CAMAC list */
    if (CRLIST(filename, list, MAXLIST, &lenlis, &i)) {
	perror("CRLIST");
        exit(2);
    }
    if (CLLOAD(list, lenlis, &i)) {
	perror("CLLOAD");
        exit(3);
    }

/**** Measurement ****/

/* time=0 */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
	CAMAC(nafrt, &dat1, &q, &x);
	printf("%d\n", dat1 - dat0);
    }

/* Q-IGNORE 24bit write */
    printf("%d\n", len);
    for (i = min; i < max; i += step) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
        status = CDMAL(1, nafw, data, i, &retlen, &err);
	CAMAC(nafrt, &dat1, &q, &x);
	if (status || err) {
	    perror("CDMAL");
	    exit(100);
	}
	else {
	    printf("%d %d\n", i, dat1 - dat0);
	}
    }

/* Q-IGNORE 24bit read */
    printf("%d\n", len);
    for (i = min; i < max; i += step) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
        status = CDMAL(1, nafr, buf, i, &retlen, &err);
	CAMAC(nafrt, &dat1, &q, &x);
	if (status || err) {
	    perror("CDMAL");
	    exit(100);
	}
	else {
	    printf("%d %d\n", i, dat1 - dat0);
	}
    }

/* Q-IGNORE 16bit write */
    printf("%d\n", len);
    for (i = min; i < max; i += step) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
        status = CDMAW(1, nafw, datas, i, &retlen, &err);
	CAMAC(nafrt, &dat1, &q, &x);
	if (status || err) {
	    perror("CDMAL");
	    exit(100);
	}
	else {
	    printf("%d %d\n", i, dat1 - dat0);
	}
    }

/* Q-IGNORE 16bit read */
    printf("%d\n", len);
    for (i = min; i < max; i += step) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
        status = CDMAW(1, nafr, bufs, i, &retlen, &err);
	CAMAC(nafrt, &dat1, &q, &x);
	if (status || err) {
	    perror("CDMAL");
	    exit(100);
	}
	else {
	    printf("%d %d\n", i, dat1 - dat0);
	}
    }

/* Execute list */
    printf("%d\n", len);
    for (i = min; i < max; i += step) {
	printf("%d ", i);
	iosb.s_reg = i;
	if (CEWAI(data, MAXDATA, &iosb)) {
	    perror("CEWAI");
	    exit(10);
	}
/*
        usp = (unsigned short *)&data[0];
	for (i = 0; i < iosb.ret_length; i++)
	  printf("%d ",usp[i]);
	printf("\n%d %d %d\n", iosb.s_reg, iosb.ret_length, iosb.status);
	exit(0);
*/
	/* time=0 */
	printf("%d ", data[1] - data[0]);

	/* IGQ 24bit read */
	j = 2;
	printf("%d ", data[j+i+1] - data[j]);

	/* IGQ 16bit read */
	j = j + i + 2;
	usp = (unsigned short *)&data[j];
	printf("%d ", usp[i+1] - usp[0]);

	/* IGQ 24bit read (double buffer) */
	j = j + i / 2 + 1;
	printf("%d ", data[j+i+1] - data[j]);

	/* IGQ 16bit read (double buffer) */
	j = j + i + 2;
	usp = (unsigned short *)&data[j];
	printf("%d\n", usp[i+1] - usp[0]);
    }

/* End */
    CAM_Close();
}
